@@ -50,7 +50,8 @@
  */
 
 #include "ldb_tdb.h"
-#include <lib/tdb_compat/tdb_compat.h>
+#include "ldb_private.h"
+#include <tdb.h>
 
 /*
   prevent memory errors on callbacks
@@ -74,13 +75,9 @@ int ltdb_err_map(enum TDB_ERROR tdb_code)
 	case TDB_ERR_IO:
 		return LDB_ERR_PROTOCOL_ERROR;
 	case TDB_ERR_LOCK:
-#ifndef BUILD_TDB2
 	case TDB_ERR_NOLOCK:
-#endif
 		return LDB_ERR_BUSY;
-#ifndef BUILD_TDB2
 	case TDB_ERR_LOCK_TIMEOUT:
-#endif
 		return LDB_ERR_TIME_LIMIT_EXCEEDED;
 	case TDB_ERR_EXISTS:
 		return LDB_ERR_ENTRY_ALREADY_EXISTS;
@@ -268,7 +265,7 @@ int ltdb_store(struct ldb_module *module, const struct ldb_message *msg, int flg
 		return LDB_ERR_OTHER;
 	}
 
-	ret = ltdb_pack_data(module, msg, &tdb_data);
+	ret = ldb_pack_data(module, msg, (struct ldb_val *)&tdb_data);
 	if (ret == -1) {
 		talloc_free(tdb_key.dptr);
 		return LDB_ERR_OTHER;
@@ -322,7 +319,7 @@ static int ltdb_add_internal(struct ldb_module *module,
 {
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 	int ret = LDB_SUCCESS;
-	unsigned int i;
+	unsigned int i, j;
 
 	for (i=0;i<msg->num_elements;i++) {
 		struct ldb_message_element *el = &msg->elements[i];
@@ -340,6 +337,22 @@ static int ltdb_add_internal(struct ldb_module *module,
 					       el->name, ldb_dn_get_linearized(msg->dn));
 			return LDB_ERR_CONSTRAINT_VIOLATION;
 		}
+
+		/* Do not check "@ATTRIBUTES" for duplicated values */
+		if (ldb_dn_is_special(msg->dn) &&
+		    ldb_dn_check_special(msg->dn, LTDB_ATTRIBUTES)) {
+			continue;
+		}
+
+		/* TODO: This is O(n^2) - replace with more efficient check */
+		for (j=0; j<el->num_values; j++) {
+			if (ldb_msg_find_val(el, &el->values[j]) != &el->values[j]) {
+				ldb_asprintf_errstring(ldb,
+						       "attribute '%s': value #%u on '%s' provided more than once",
+						       el->name, j, ldb_dn_get_linearized(msg->dn));
+				return LDB_ERR_ATTRIBUTE_OR_VALUE_EXISTS;
+			}
+		}
 	}
 
 	ret = ltdb_store(module, msg, TDB_INSERT);
@@ -668,7 +681,7 @@ int ltdb_modify_internal(struct ldb_module *module,
 		return LDB_ERR_OTHER;
 	}
 
-	tdb_data = tdb_fetch_compat(ltdb->tdb, tdb_key);
+	tdb_data = tdb_fetch(ltdb->tdb, tdb_key);
 	if (!tdb_data.dptr) {
 		talloc_free(tdb_key.dptr);
 		return ltdb_err_map(tdb_error(ltdb->tdb));
@@ -681,7 +694,7 @@ int ltdb_modify_internal(struct ldb_module *module,
 		goto done;
 	}
 
-	ret = ltdb_unpack_data(module, &tdb_data, msg2);
+	ret = ldb_unpack_data(ldb_module_get_ctx(module), (struct ldb_val *)&tdb_data, msg2);
 	free(tdb_data.dptr);
 	if (ret == -1) {
 		ret = LDB_ERR_OTHER;
@@ -765,6 +778,7 @@ int ltdb_modify_internal(struct ldb_module *module,
 
 				/* Check that values don't exist yet on multi-
 				   valued attributes or aren't provided twice */
+				/* TODO: This is O(n^2) - replace with more efficient check */
 				for (j = 0; j < el->num_values; j++) {
 					if (ldb_msg_find_val(el2, &el->values[j]) != NULL) {
 						if (control_permissive) {
@@ -977,9 +991,12 @@ static int ltdb_modify(struct ltdb_context *ctx)
 static int ltdb_rename(struct ltdb_context *ctx)
 {
 	struct ldb_module *module = ctx->module;
+	void *data = ldb_module_get_private(module);
+	struct ltdb_private *ltdb = talloc_get_type(data, struct ltdb_private);
 	struct ldb_request *req = ctx->req;
 	struct ldb_message *msg;
 	int ret = LDB_SUCCESS;
+	TDB_DATA tdb_key, tdb_key_old;
 
 	ldb_request_set_state(req, LDB_ASYNC_PENDING);
 
@@ -992,25 +1009,58 @@ static int ltdb_rename(struct ltdb_context *ctx)
 		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
-	/* in case any attribute of the message was indexed, we need
-	   to fetch the old record */
+	/* we need to fetch the old record to re-add under the new name */
 	ret = ltdb_search_dn1(module, req->op.rename.olddn, msg);
 	if (ret != LDB_SUCCESS) {
 		/* not finding the old record is an error */
 		return ret;
 	}
 
+	/* We need to, before changing the DB, check if the new DN
+	 * exists, so we can return this error to the caller with an
+	 * unmodified DB */
+	tdb_key = ltdb_key(module, req->op.rename.newdn);
+	if (!tdb_key.dptr) {
+		talloc_free(msg);
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
+	tdb_key_old = ltdb_key(module, req->op.rename.olddn);
+	if (!tdb_key_old.dptr) {
+		talloc_free(msg);
+		talloc_free(tdb_key.dptr);
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
+	/* Only declare a conflict if the new DN already exists, and it isn't a case change on the old DN */
+	if (tdb_key_old.dsize != tdb_key.dsize || memcmp(tdb_key.dptr, tdb_key_old.dptr, tdb_key.dsize) != 0) {
+		if (tdb_exists(ltdb->tdb, tdb_key)) {
+			talloc_free(tdb_key_old.dptr);
+			talloc_free(tdb_key.dptr);
+			ldb_asprintf_errstring(ldb_module_get_ctx(module),
+					       "Entry %s already exists",
+					       ldb_dn_get_linearized(msg->dn));
+			/* finding the new record already in the DB is an error */
+			talloc_free(msg);
+			return LDB_ERR_ENTRY_ALREADY_EXISTS;
+		}
+	}
+	talloc_free(tdb_key_old.dptr);
+	talloc_free(tdb_key.dptr);
+
 	/* Always delete first then add, to avoid conflicts with
 	 * unique indexes. We rely on the transaction to make this
 	 * atomic
 	 */
 	ret = ltdb_delete_internal(module, msg->dn);
 	if (ret != LDB_SUCCESS) {
+		talloc_free(msg);
 		return ret;
 	}
 
 	msg->dn = ldb_dn_copy(msg, req->op.rename.newdn);
 	if (msg->dn == NULL) {
+		talloc_free(msg);
 		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
@@ -1020,6 +1070,8 @@ static int ltdb_rename(struct ltdb_context *ctx)
 	 */
 	ret = ltdb_add_internal(module, msg, false);
 
+	talloc_free(msg);
+
 	return ret;
 }
 
